package javax.swing;

import java.awt.IllegalComponentStateException;
import java.util.Locale;
import javax.accessibility.AccessibleStateSet;
import javax.accessibility.AccessibleRole;
import javax.accessibility.AccessibleIcon;
import java.lang.reflect.Field;
import javax.accessibility.AccessibleContext;
import java.awt.image.PixelGrabber;
import java.io.ObjectOutputStream;
import java.io.IOException;
import java.awt.image.MemoryImageSource;
import java.awt.image.ColorModel;
import java.io.ObjectInputStream;
import java.awt.Graphics;
import java.beans.Transient;
import sun.awt.AppContext;
import java.beans.ConstructorProperties;
import java.awt.Toolkit;
import java.security.AccessController;
import java.security.AccessControlContext;
import java.security.ProtectionDomain;
import java.security.PrivilegedAction;
import java.awt.MediaTracker;
import java.awt.Component;
import java.awt.image.ImageObserver;
import java.awt.Image;
import java.net.URL;
import javax.accessibility.Accessible;
import java.io.Serializable;

public class ImageIcon implements Icon, Serializable, Accessible
{
    private transient String filename;
    private transient URL location;
    transient Image image;
    transient int loadStatus;
    ImageObserver imageObserver;
    String description;
    @Deprecated
    protected static final Component component;
    @Deprecated
    protected static final MediaTracker tracker;
    private static int mediaTrackerID;
    private static final Object TRACKER_KEY;
    int width;
    int height;
    private ImageIcon.AccessibleImageIcon accessibleContext;
    
    private static Component createNoPermsComponent() {
        return (Component)AccessController.doPrivileged(new PrivilegedAction<Component>() {
            @Override
            public Component run() {
                return new Component() {};
            }
        }, new AccessControlContext(new ProtectionDomain[] { new ProtectionDomain(null, null) }));
    }
    
    public ImageIcon(String filename, String description) {
        this.loadStatus = 0;
        this.description = null;
        this.width = -1;
        this.height = -1;
        this.accessibleContext = null;
        this.image = Toolkit.getDefaultToolkit().getImage(filename);
        if (this.image == null) {
            return;
        }
        this.filename = filename;
        this.description = description;
        this.loadImage(this.image);
        return;
    }
    
    @ConstructorProperties({ "description" })
    public ImageIcon(String s) {
        this(s, s);
    }
    
    public ImageIcon(URL location, String description) {
        this.loadStatus = 0;
        this.description = null;
        this.width = -1;
        this.height = -1;
        this.accessibleContext = null;
        this.image = Toolkit.getDefaultToolkit().getImage(location);
        if (this.image == null) {
            return;
        }
        this.location = location;
        this.description = description;
        this.loadImage(this.image);
        return;
    }
    
    public ImageIcon(URL url) {
        this(url, url.toExternalForm());
    }
    
    public ImageIcon(Image image, String description) {
        this(image);
        this.description = description;
    }
    
    public ImageIcon(Image image) {
        this.loadStatus = 0;
        this.description = null;
        this.width = -1;
        this.height = -1;
        this.accessibleContext = null;
        this.image = image;
        Object property = image.getProperty("comment", this.imageObserver);
        if ((property instanceof String)) {
            this.description = (String)property;
        }
        this.loadImage(image);
    }
    
    public ImageIcon(byte[] imagedata, String description) {
        this.loadStatus = 0;
        this.description = null;
        this.width = -1;
        this.height = -1;
        this.accessibleContext = null;
        this.image = Toolkit.getDefaultToolkit().createImage(imagedata);
        if (this.image == null) {
            return;
        }
        this.description = description;
        this.loadImage(this.image);
        return;
    }
    
    public ImageIcon(byte[] imagedata) {
        this.loadStatus = 0;
        this.description = null;
        this.width = -1;
        this.height = -1;
        this.accessibleContext = null;
        this.image = Toolkit.getDefaultToolkit().createImage(imagedata);
        if (this.image == null) {
            return;
        }
        Object property = this.image.getProperty("comment", this.imageObserver);
        if ((property instanceof String)) {
            this.description = (String)property;
        }
        this.loadImage(this.image);
        return;
    }
    
    public ImageIcon() {
        this.loadStatus = 0;
        this.description = null;
        this.width = -1;
        this.height = -1;
        this.accessibleContext = null;
    }
    
    protected void loadImage(Image image) {
        MediaTracker tracker = this.getTracker();
        synchronized (tracker) {
            int nextID = this.getNextID();
            tracker.addImage(image, nextID);
            try {
                tracker.waitForID(nextID, 0L);
            }
            catch (InterruptedException ex) {
                System.out.println("INTERRUPTED while loading Image");
            }
            this.loadStatus = tracker.statusID(nextID, false);
            tracker.removeImage(image, nextID);
            this.width = image.getWidth(this.imageObserver);
            this.height = image.getHeight(this.imageObserver);
        }
    }
    
    private int getNextID() {
        synchronized (this.getTracker()) {
            return ++ImageIcon.mediaTrackerID;
        }
    }
    
    private MediaTracker getTracker() {
        AppContext appContext = AppContext.getAppContext();
        Object value;
        synchronized (appContext) {
            value = appContext.get(ImageIcon.TRACKER_KEY);
            if (value == null) {
                value = new MediaTracker(new Component() {});
                appContext.put(ImageIcon.TRACKER_KEY, value);
            }
        }
        return (MediaTracker)value;
    }
    
    public int getImageLoadStatus() {
        return this.loadStatus;
    }
    
    @Transient
    public Image getImage() {
        return this.image;
    }
    
    public void setImage(Image image) {
        this.loadImage(this.image = image);
    }
    
    public String getDescription() {
        return this.description;
    }
    
    public void setDescription(String description) {
        this.description = description;
    }
    
    @Override
    public synchronized void paintIcon(Component component, Graphics graphics, int n, int n2) {
        if (this.imageObserver == null) {
            graphics.drawImage(this.image, n, n2, component);
        }
        else {
            graphics.drawImage(this.image, n, n2, this.imageObserver);
        }
    }
    
    @Override
    public int getIconWidth() {
        return this.width;
    }
    
    @Override
    public int getIconHeight() {
        return this.height;
    }
    
    public void setImageObserver(ImageObserver imageObserver) {
        this.imageObserver = imageObserver;
    }
    
    @Transient
    public ImageObserver getImageObserver() {
        return this.imageObserver;
    }
    
    @Override
    public String toString() {
        if (this.description != null) {
            return this.description;
        }
        return super.toString();
    }
    
    private void readObject(ObjectInputStream objectInputStream) throws ClassNotFoundException, IOException {
        objectInputStream.defaultReadObject();
        int int1 = objectInputStream.readInt();
        int int2 = objectInputStream.readInt();
        int[] pix = (int[])(int[])objectInputStream.readObject();
        if (pix != null) {
            this.loadImage(this.image = Toolkit.getDefaultToolkit().createImage(new MemoryImageSource(int1, int2, ColorModel.getRGBdefault(), pix, 0, int1)));
        }
    }
    
    private void writeObject(ObjectOutputStream objectOutputStream) throws IOException {
        objectOutputStream.defaultWriteObject();
        int iconWidth = this.getIconWidth();
        int iconHeight = this.getIconHeight();
        int[] array = (this.image != null) ? new int[iconWidth * iconHeight] : null;
        if (this.image != null) {
            try {
                PixelGrabber o = new PixelGrabber(this.image, 0, 0, iconWidth, iconHeight, array, 0, iconWidth);
                o.grabPixels();
                if ((o.getStatus() & 0x80) != 0x0) {
                    throw new IOException("failed to load image contents");
                }
            }
            catch (InterruptedException ex) {
                throw new IOException("image load interrupted");
            }
        }
        objectOutputStream.writeInt(iconWidth);
        objectOutputStream.writeInt(iconHeight);
        objectOutputStream.writeObject(array);
    }
    
    @Override
    public AccessibleContext getAccessibleContext() {
        if (this.accessibleContext == null) {
            this.accessibleContext = new ImageIcon.AccessibleImageIcon();
        }
        return this.accessibleContext;
    }
    
    static Component access$000() {
        return createNoPermsComponent();
    }
    
    static {
        component = (Component)AccessController.doPrivileged(new PrivilegedAction<Component>() {
            @Override
            public Component run() {
                try {
                    Component access$000 = ImageIcon.access$000();
                    Field declaredField = Component.class.getDeclaredField("appContext");
                    declaredField.setAccessible(true);
                    declaredField.set(access$000, null);
                    return access$000;
                }
                catch (Throwable t) {
                    t.printStackTrace();
                    return null;
                }
            }
        });
        tracker = new MediaTracker(ImageIcon.component);
        TRACKER_KEY = new StringBuilder("TRACKER_KEY");
    }
    
    protected class AccessibleImageIcon extends AccessibleContext implements AccessibleIcon, Serializable
    {
        @Override
        public AccessibleRole getAccessibleRole() {
            return AccessibleRole.ICON;
        }
        
        @Override
        public AccessibleStateSet getAccessibleStateSet() {
            return null;
        }
        
        @Override
        public Accessible getAccessibleParent() {
            return null;
        }
        
        @Override
        public int getAccessibleIndexInParent() {
            return -1;
        }
        
        @Override
        public int getAccessibleChildrenCount() {
            return 0;
        }
        
        @Override
        public Accessible getAccessibleChild(int n) {
            return null;
        }
        
        @Override
        public Locale getLocale() throws IllegalComponentStateException {
            return null;
        }
        
        @Override
        public String getAccessibleIconDescription() {
            return ImageIcon.this.getDescription();
        }
        
        @Override
        public void setAccessibleIconDescription(String description) {
            ImageIcon.this.setDescription(description);
        }
        
        @Override
        public int getAccessibleIconHeight() {
            return ImageIcon.this.height;
        }
        
        @Override
        public int getAccessibleIconWidth() {
            return ImageIcon.this.width;
        }
        
        private void readObject(ObjectInputStream objectInputStream) throws ClassNotFoundException, IOException {
            objectInputStream.defaultReadObject();
        }
        
        private void writeObject(ObjectOutputStream objectOutputStream) throws IOException {
            objectOutputStream.defaultWriteObject();
        }
    }
}
